//
//  BuiltInConsoleController.m
//  R
//
//  Created by Byron Ellis on 2/28/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "BuiltInConsoleController.h"
#import "RInterpreter.h"

@implementation BuiltInConsoleController

- (void)awakeFromNib {
	[super awakeFromNib];
	
	NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
	[dc addObserver:self selector:@selector(consoleWillChange:) name:RInterpreterWillChangeConsole object:nil];
	activeConsole = NO;

	//We need to update our fonts and colors
	RInterpreter *ri = [RInterpreter sharedInterpreter];
	[ri setPromptAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
		[NSFont userFixedPitchFontOfSize:12.0],NSFontAttributeName,
		[NSColor blueColor],NSForegroundColorAttributeName,nil]];
	[ri setInputAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
		[NSFont userFixedPitchFontOfSize:12.0],NSFontAttributeName,nil]];
	[ri setOutputAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
		[NSFont userFixedPitchFontOfSize:12.0],NSFontAttributeName,nil]];
		
	
	
	[ri setConsoleDocument:self 
		  		    buffer:(CFMutableAttributedStringRef)[self textStorage]
					  copy:[[RInterpreter sharedInterpreter] hasActiveConsole] ? NO : YES];
	
}	

#pragma mark Evaluation Tracking

- (void)evalWillStart:(id)sender {
	[textView setEditable:NO];
}
- (void)evalDidFinish:(id)sender {
	[textView setEditable:YES];
}



#pragma mark Console Interaction

- (void)didWritePrompt:(id)sender {
	NSInteger inputStart = [[self textStorage] length];
	self.lineStart = inputStart;

	[textView scrollRangeToVisible:NSMakeRange(inputStart,0)];
	[textView setTypingAttributes:[[RInterpreter sharedInterpreter] inputAttributes]];
	NSLog(@"Prompt now at: %ld",(long)inputStart);
}

- (void)willReadFromConsole:(id)sender {
	self.hinting = YES;
}
- (void)didReadFromConsole:(id)sender {
	self.hinting = NO;
}

- (void)consoleWillChange:(NSNotification*)aNotification {
	NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
	if([aNotification object] == self) {
		NSLog(@"Marking ourselves editable");
		[dc addObserver:self selector:@selector(didWritePrompt:) name:RInterpreterDidWritePrompt object:nil];
		[dc addObserver:self selector:@selector(willReadFromConsole:) name:RInterpreterWillReadFromConsole object:nil];
		[dc addObserver:self selector:@selector(didReadFromConsole:) name:RInterpreterDidReadFromConsole object:nil];
		[dc addObserver:self selector:@selector(evalWillStart:) name:RInterpreterWillStartEvaluating object:nil];
		[dc addObserver:self selector:@selector(evalDidFinish:) name:RInterpreterDidFinishEvaluating object:nil];
		
		[textView setEditable:YES];		
		activeConsole = YES;
		
		//Build a set of text attributes and tell the console manager about them.		
	} else {
		NSLog(@"Marking ourselves not-editable");
		[dc removeObserver:self	name:RInterpreterDidWritePrompt object:nil];
		[dc removeObserver:self name:RInterpreterWillReadFromConsole object:nil];
		[dc removeObserver:self name:RInterpreterDidReadFromConsole object:nil];
		[dc removeObserver:self name:RInterpreterWillStartEvaluating object:nil];
		[dc removeObserver:self name:RInterpreterDidFinishEvaluating object:nil];
		[textView setEditable:NO];
		activeConsole = NO;
		
	}
}



#pragma mark TextView Delegate


- (BOOL)textView:(NSTextView*)aTextView shouldChangeTextInRange:(NSRange)aRange replacementString:(NSString*)aString {
	if(nil != aString && (aRange.location < lineStart || NO == activeConsole)) return NO;
	return YES;
}


- (BOOL)textView:(NSTextView*)aTextView doCommandBySelector:(SEL)aSelector {
	NSRange aRange = [aTextView selectedRange];
	if(@selector(insertNewline:) == aSelector) {
		NSInteger len = [[aTextView textStorage] length];		
		[aTextView setSelectedRange:NSMakeRange(len,0)];		
		[aTextView insertText:@"\n"];			
		NSString *aString = [[aTextView attributedSubstringFromRange:NSMakeRange(lineStart,len+1)] string];
		RInterpreter *ri = [RInterpreter sharedInterpreter];
		if(YES == [ri shouldAddToHistory]) {
		}
		[ri appendInput:aString evaluate:YES];
		return YES;
	} else if(@selector(moveUp:) == aSelector) {
		NSInteger len = [[aTextView textStorage] length];
		if(aRange.location == lineStart || aRange.location == len) {
			return YES;
		}
	} else if(@selector(moveDown:) == aSelector) {
		NSInteger len = [[aTextView textStorage] length];
		if(aRange.location == lineStart || aRange.location == len) {
			return YES;
		}
	} else if(@selector(insertTab:) == aSelector) {
		[aTextView complete:self];
		return YES;
	} else if(@selector(moveLeft:) == aSelector || @selector(moveWordLeft:) == aSelector || 
	          @selector(moveLeftAndModifySelection:) == aSelector || @selector(moveWordLeftAndModifySelection:) == aSelector) {
		if(aRange.location == lineStart) return YES;
		aRange.location++;
	} else if(@selector(moveToBeginningOfParagraph:) == aSelector || @selector(moveToBeginningOfLine:) == aSelector) {
		[aTextView setSelectedRange:NSMakeRange(lineStart,0)];
		return YES;
	} else if(@selector(moveToBeginningOfParagraphAndModifySelection:) == aSelector || @selector(moveToBeginningOfLineAndModifySelection:) == aSelector) {
		[aTextView setSelectedRange:NSMakeRange(lineStart,(aRange.location+aRange.length)-lineStart)];
		return YES;
	} 
	return [super textView:aTextView doCommandBySelector:aSelector];
}

- (NSString*)windowTitleForDocumentDisplayName:(NSString*)displayName {
	if(YES == activeConsole)
		return @"Console (Active)";
	else
		return @"Console (Inactive)";
}


@end
