#import "RApplicationDelegate.h"
#import "RInterpreter.h"
#import "RDocumentController.h"

@implementation RApplicationDelegate

- (IBAction)newConsoleDocument:(id)sender {
	NSError *error = nil;
	RDocumentController *dc = [RDocumentController sharedDocumentController];
	NSDocument *console = [dc makeConsoleDocumentOfType:[dc principalConsoleDocumentType] error:&error];
	if(nil == error) {
		[dc addDocument:console];
		[console makeWindowControllers];
		[console showWindows];
	} else 
		NSLog([error description]);
}

- (BOOL)applicationShouldOpenUntitledFile:(NSApplication*)sender {
	return NO;
}


- (void)applicationWillFinishLaunching:(NSNotification*)aNotification {
	//Make sure OUR document controller gets the nod.
	[RDocumentController sharedDocumentController];
	
	//Set our home directory and our user package directory and other things we should pick up from preferences.
	RInterpreter *interp = [RInterpreter sharedInterpreter];
	[interp setHomeDirectory:[[NSBundle bundleWithIdentifier:@"org.r-project.R-framework"] resourcePath]];
	//Detect ESS (or just plain EMACS I suppose) and switch the interpreter to being "driven" from stdin/stdout if this is the case.
	if(NULL != getenv("EMACS") && strncmp(getenv("EMACS"),"t",1)==0) {
		[interp setUsesStandardReader:YES];
		//Perhaps we should also write some code to hide ourselves on the dock if this is the case? I don't know
		//if we can do this programatically though...
		
		//We should definitely suppress the ability to create consoles from within R.app (turn off the appropriate menu items and so on).
	}
}

- (void)applicationDidFinishLaunching:(NSNotification*)aNotification {
	[[NSNotificationCenter defaultCenter] addObserver:self
										  selector:@selector(interpreterDidFinishLaunching:)
											  name:RInterpreterDidFinishStartingInterpreter
											object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
										  selector:@selector(interpreterCreateDevice:)
											  name:RInterpreterOpenDevice
											object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
										  selector:@selector(interpreterShowHelpForFunction:)
											  name:RInterpreterShowHelpForFunction
											object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
										  selector:@selector(interpreterShowHelpFiles:)
											name:RInterpreterShowHelpFiles
											object:nil];
	[[RInterpreter sharedInterpreter] stop];
}	

- (void)interpreterShowHelpForFunction:(NSNotification*)aNotification {
}

- (void)interpreterShowHelpFiles:(NSNotification*)aNotification {
	NSArray *files = [aNotification object];
	RDocumentController *rc = [RDocumentController sharedDocumentController];
}

- (void)interpreterDidFinishLaunching:(NSNotification*)aNotification {
	//If the "standard reader" is being used, assume that we are in ESS mode and
	//that the Console will be driven from stdin/stdout. We'll still get notifications
	//for systems that use it, but hopefully not much else.
	if([[RInterpreter sharedInterpreter] usesStandardReader] == YES) return;

	//Create our stored procedure for opening help for our system
	
	//When the interpreter is done starting up try to get the console (it may have
	//changed during startup).	
	[self newConsoleDocument:nil];	
}

- (void)interpreterCreateDevice:(NSNotification*)aNotify {
	RDocumentController *dc     = [RDocumentController sharedDocumentController];
	NSError             *error  = nil;
//	NSLog(@"Trying to create device");
	NSDocument          *device = [dc makeDeviceDocumentOfType:[dc deviceDocumentTypeForDisplay:[[aNotify object] objectForKey:@"Display"]] error:&error];
	if(nil == error) {
//		NSLog(@"Creating device: %@",[device description]);
		if([device respondsToSelector:@selector(linkWithDeviceDescription:)]) {
			[dc addDocument:device];
			[device makeWindowControllers];
			[device linkWithDeviceDescription:[aNotify object]];		
			NSWindow *key = [NSApp keyWindow];
			[device showWindows];
			if(nil != key) [key makeKeyAndOrderFront:self];
		}
	}
}
@end
