//
//  RHintingController.m
//  R
//
//  Created by Byron Ellis on 2/28/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "RHintingController.h"
#import "RInterpreter.h"
#import "REnvironment.h"

@implementation RHintingController

@synthesize hinting,lineStart;

- (NSTextStorage*)textStorage { return [textView textStorage]; }

- (IBAction)helpButton:(id)sender {
	//This shouldn't be active with the function name set to nil, but we
	//check just in case.
	if(nil == hintFunName) return;
	//Otherwise, talk to the help system
	[[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:RInterpreterShowHelpForFunction object:hintFunName]];
}

- (void)awakeFromNib {
	hinting = YES;
	
	//No spell checking.
	[textView setContinuousSpellCheckingEnabled:NO];
	
	NSError *error = nil;
	hintExpr = [[RValue valueWithString:@"gsub('\\\\s','',paste(capture.output(print(args(._)))[1:2],collapse=''))"] parseWithError:&error];
	if(nil != error)
		hintExpr = nil;
	hintFunName  = nil;
	hintAttrs    = [NSDictionary dictionaryWithObjectsAndKeys:[[NSColor greenColor] blendedColorWithFraction:0.5 ofColor:[NSColor blackColor]],NSUnderlineColorAttributeName,
					[NSNumber numberWithInt:NSUnderlineStyleThick|NSUnderlinePatternDot],NSUnderlineStyleAttributeName,nil];
	currentFn     = NSMakeRange(0,0);
	deleteBackward= NO;	
	
	//rcompgen support for completions (if available)
	compExpr = [[RValue valueWithString:@"{rcompgen:::.assignLinebuffer(._)\nrcompgen:::.assignEnd(._)\nrcompgen:::.guessTokenFromLine()\nrcompgen:::.completeToken()\nrcompgen:::.retrieveCompletions()}"] parseWithError:&error];
	if(nil != error)
		compExpr = nil;
	
	[[textView textStorage] setDelegate:self];	
}

//TODO: Consider using an NSOperationQueue to handle the process of updating the function name
//to avoid blocking, though I've never noticed it being terribly instrusive in terms of lookup time.
- (void)setHintFunction:(NSString*)aFunName {
	if(nil == hintExpr || nil == aFunName) {
		hintFunName = nil;
		[helpButton setEnabled:NO];
		[statusText setStringValue:@""];
		return;
	}
	[hintExpr setArgument:0 value:[RValue valueWithString:aFunName]];
	RValue *result = [[REnvironment global] evaluate:hintExpr blocking:NO];
	if(nil != result) {
		NSMutableString *temp = [NSMutableString stringWithFormat:@"%@%@",aFunName,[result stringValue]];
		@try {
			[temp replaceCharactersInRange:NSMakeRange([aFunName length],[@"function" length]) withString:@""];
			[statusText setStringValue:temp];
			hintFunName = aFunName;
			[helpButton setEnabled:YES];
		} @catch(NSException *e) {
			hintFunName = nil;
			[helpButton setEnabled:NO];
			[statusText setStringValue:@""];			
		}
		return;
	} else {
		hintFunName = nil;
		[helpButton setEnabled:NO];
		[statusText setStringValue:@""];
	}
}

- (void)showHelp:(id)sender {
	NSLog(@"Showing help for current function");
	[self helpButton:sender];
}

#pragma mark TextStorage delegate
#pragma mark -

- (NSRange)markFunctionFrom:(NSRange)r inString:(NSString*)s {
	NSInteger index = r.location;NSInteger loc;
	unichar c;
	
	NSInteger parenCount = 0;
	if(index >= [s length]) index = [s length]-1;
	
	for(;index > lineStart;index--) {
		c = [s characterAtIndex:index];
		//If we are at an open paren and the "last" paren we can find
		if(c == '(')
			if(parenCount == 0) break; else parenCount--;
		if(c == ')') parenCount++;
	}
	if(index <= lineStart) return NSMakeRange(0,0);
	//If we're at an open paren, move down one.
	if(c == '(') { index--;c = [s characterAtIndex:index]; }
		if(c == '"') { index--;c = [s characterAtIndex:index]; }
		loc = index;
	while( (c >= '0' && c <= '9') ||
		  (c >= 'a' && c <= 'z') ||
		  (c >= 'A' && c <= 'Z') ||
		  (c == '.' || c == '_') ) {
		if(--index <= lineStart) break;
		c = [s characterAtIndex:index];
	}
	//We overshot?
	if(index > lineStart) index++;
	return (loc-index > 0) ? NSMakeRange(index,1+(loc-index)) : NSMakeRange(0,0);
}

- (void)markFunctionIn:(NSTextStorage*)ts withRange:(NSRange)aRange {
	NSString *s        = [ts string];
	NSInteger       len      = [s length];
	NSRange   newRange = [self markFunctionFrom:aRange inString:s];
	//Don't change things if the function hasn't changed.
	if(newRange.location != 0 && newRange.location == currentFn.location && newRange.length == currentFn.length)
		return;
	//Remove the present function attributes
	NSLayoutManager *lm = [textView layoutManager];
	if(currentFn.location > 0) {
		if(currentFn.location+currentFn.length >= len) {
			currentFn.length = (len - currentFn.location);
		}
		[lm removeTemporaryAttribute:NSUnderlineStyleAttributeName forCharacterRange:currentFn];
		[lm removeTemporaryAttribute:NSForegroundColorAttributeName forCharacterRange:currentFn];
	}
	currentFn = newRange;
	if(currentFn.location > 0) {
		if(currentFn.location+currentFn.length >= len) {
			currentFn.length = (len - currentFn.location);
		}
		[lm addTemporaryAttributes:hintAttrs forCharacterRange:currentFn];	
		[self setHintFunction:[s substringWithRange:currentFn]];
	} else [self setHintFunction:nil];
}

- (void)textStorageDidProcessEditing:(NSNotification*)aN {
	if(NO == deleteBackward && YES == hinting) 
		[self markFunctionIn:(NSTextStorage*)[aN object] withRange:[textView selectedRange]];
	deleteBackward = NO;
}

#pragma mark TextView delegate
#pragma mark -
- (BOOL)textView:(NSTextView*)aTextView doCommandBySelector:(SEL)aSelector {
	if(@selector(deleteBackward:) == aSelector) 
		deleteBackward = YES;
	if(NO == deleteBackward && YES == hinting) {
		NSRange aRange = [aTextView selectedRange];
		[self markFunctionIn:[aTextView textStorage] withRange:aRange];
	}
	return NO;
}

- (NSArray*)textView:(NSTextView*)aTextView completions:(NSArray*)words forPartialWordRange:(NSRange)charRange indexOfSelectedItem:(NSInteger*)index {
	if(nil != compExpr) {
		[compExpr setArgument:0 value:[RValue valueWithString:[[[aTextView textStorage] string] substringFromIndex:lineStart]]];		
		[compExpr setArgument:1 value:[RValue valueWithInteger:(charRange.location+charRange.length)-lineStart]];
		RValue *result = [[REnvironment global] evaluate:compExpr blocking:NO];
		if(nil != result && [result length] > 0 && [result isStringType]) {
			NSLog(@"compExpr worked!");
			return [result arrayValue];
		}
	}
	
	return nil;
}


@end
